function [yhat,Xfit,E,R2X,Q2,RMSEP]=cp3pred(Xnew,ynew,n2,m,p,W,B,C,Py,ymean,Xmean,ysdev,Xsdev);

% function [yhat,Xfit,E,R2X,Q2,RMSEP]=cp3pred(Xnew,ynew,n2,m,p,W,B,C,Py,ymean,Xmean,ysdev,Xsdev);
% This function produces predictions using a PARAFAC/PCovR model
% It is assumed that data are always mean centered and y is univariate
%
% INput: Xnew: n2*m*p array of the n2 new objects; p frontal slices
%              (n2*m) next to each other
%        n2,m,p: order of Xnew
%        ynew: n2*1 vector of known y-values in test set
%        ymean: (q*1) vector of training y-means
%        Xmean: (mp) vector of training X-means
%        ysdev: (q*1)vector of training sdev values
%        Xsdev: (mp) vector of training sdev values
%        W: (mp*r) weight matrix (XW=A) from training set; 
%        r is number of parafac components
%    B (m*r),C (p*r): X loadings from training set
%    Py: (r*q) matrix of regression coeffients for Y (training set)
%
%
% Output: yhat: (n2*q) matrix of predicted Y values
%         Xfit: (n2*m*p) array of fitted X values; p frontal slices (n2*m) next
%               to each other  
%         E: (n2*mp) array of X residuals (frontal slices next to each other)
%     R2X: number between 0 and 100% indicating what part of Xnew is fitted
%         Q2: number lower than 100% indicating what part of Ynew is predicted
%         RMSEP: root mean squared error of prediction in test set
%
% Age K. Smilde, 11-4-1997
%   ==========================================================================
%   Copyright 2005 Biosystems Data Analysis Group ; Universiteit van Amsterdam
%   ==========================================================================


% Scale the new matrix Xnew and Ynew using the training set scaling constants

            
if ( nargin ==11 )
   Xn=scale(Xnew,Xmean);
elseif ( nargin ==13 )
   Xn=scale(Xnew,Xmean,Xsdev);
else
   Xn=Xnew;
end;


% Predict y (calculate yhat)

yhatraw=Xn*W*Py;

% Correct the predictions for scaling

            
if ( nargin == 11 )
   yhat=rescale(yhatraw,ymean);
elseif ( nargin == 13 )
   yhat=rescale(yhatraw,ymean,ysdev);
else
   yhat=yhatraw;
end;


% Calculate raw fitted values of Xn

Xfitraw=zeros(size(Xn));
for k=1:p
    Xfitraw(:,(k-1)*m+1:(k*m))=Xn*W*diag(C(k,:))*B';
end;

% Correct the raw fitted values for scaling

            
if ( nargin == 11 )
   Xfit=rescale(Xfitraw,Xmean);
elseif ( nargin == 13 )
   Xfit=rescale(Xfitraw,Xmean,Xsdev);
else
   Xfit=Xfitraw;
end;


% Calculate residuals and PercXfit                     

   E=Xn-Xfitraw;

%  E=Xnew-Xfit;
%  Xn=scale(Xnew,Xmean);

  SSX=ssq(Xn);
  SSEX=ssq(E);
%  disp('R2X is')
  R2X=100*(SSX-SSEX)/SSX;

  SSY=ssq(ynew-ymean);
  SSEY=ssq(yhat-ynew);
%  disp('Q2 is')
  Q2=100*(SSY-SSEY)/SSY;

  RMSEP=sqrt(SSEY/n2);